#!/usr/bin/env python3

# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import math
from datetime import datetime

import pandas as pd


def ts_to_xminute(x, ts):
    """function to drop a timestamp down to the nearest x minute"""
    return datetime(ts.year, ts.month, ts.day, ts.hour, x * math.floor(ts.minute / x))


def ts_to_xsecond(x, ts):
    """function to drop a timestamp down to the nearest x second"""
    return datetime(ts.year, ts.month, ts.day, ts.hour, ts.minute, x * math.floor(ts.second / x))


def ts_to_1second(ts):
    return ts_to_xsecond(1, ts)


def ts_to_2second(ts):
    return ts_to_xsecond(2, ts)


def ts_to_15second(ts):
    return ts_to_xsecond(15, ts)


def ts_to_1minute(ts):
    return ts_to_xminute(1, ts)


def ts_to_2minute(ts):
    return ts_to_xminute(2, ts)


def ts_to_5minute(ts):
    return ts_to_xminute(5, ts)


def ts_to_10minute(ts):
    return ts_to_xminute(10, ts)


def ts_to_15minute(ts):
    return ts_to_xminute(15, ts)


def ts_to_30minute(ts):
    return ts_to_xminute(30, ts)


def df_split(df, chunk_size=65536, itr=False):
    """break a dataframe into chunks"""
    if itr:
        lst_or_itr = (df.iloc[i : i + chunk_size].copy() for i in range(0, df.shape[0], chunk_size))
    else:
        lst_or_itr = [df.iloc[i : i + chunk_size].copy() for i in range(0, df.shape[0], chunk_size)]
    return lst_or_itr


def df_join(df_lst):
    """join a dataframe from chunks."""
    df = pd.concat(df_lst)
    return df


def df_groupby_agg(df, by=None, sort=True, agg_c_dct=None, agg_f_dct=None, chunk_size=524288):
    """
    groupby a dataframe in chunks, then recombine.
    agg_c_dct is for the chunk grouping
    agg_f_dct is for the final grouping
    """
    df_lst = []
    itr = df_split(df, chunk_size=chunk_size, itr=True)
    for dfc in itr:
        df_lst.append(dfc.groupby(by, as_index=False, sort=sort).agg(agg_c_dct))
    dfl = df_join(df_lst)
    dfa = dfl.groupby(by, as_index=False, sort=sort).agg(agg_f_dct)
    return dfa
