# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

# pylint: disable=missing-module-docstring
# pylint: disable=missing-class-docstring
# pylint: disable=missing-function-docstring

import dataclasses
from importlib.abc import (
    ExecutionLoader as _EmbeddedImporter__abc_ExecutionLoader,
    MetaPathFinder as _EmbeddedImporter__abc_MetaPathFinder,
)


class _EmbeddedImporter(_EmbeddedImporter__abc_ExecutionLoader, _EmbeddedImporter__abc_MetaPathFinder):
    # pylint: disable=import-outside-toplevel
    from importlib.machinery import ModuleSpec
    import sys
    from typing import Dict

    @dataclasses.dataclass
    class RegisteredModule:
        fullname: str
        is_package: bool
        filename: str
        source_text: str
        source_hash: str

    def __init__(self) -> None:
        super().__init__()
        self._registered_modules: _EmbeddedImporter.Dict[str, _EmbeddedImporter.RegisteredModule] = {}

    def find_spec(self, fullname: str, path=None, target=None):  # pylint: disable=unused-argument
        """
        Return a ModuleSpec object for object for an embedded module
        """
        try:
            rm = self._registered_modules[fullname]
        except KeyError:
            return None
        module_spec = self.ModuleSpec(fullname, self, is_package=rm.is_package, origin=f"embedded {rm.filename}")
        # module_spec.has_location = False
        return module_spec

    def is_package(self, fullname: str) -> bool:
        try:
            return self._registered_modules[fullname].is_package
        except KeyError as exc:
            raise ImportError from exc

    def get_source(self, fullname: str) -> str:
        try:
            return self._registered_modules[fullname].source_text
        except KeyError as exc:
            raise ImportError from exc

    def get_filename(self, fullname: str) -> str:
        try:
            return self._registered_modules[fullname].filename
        except KeyError as exc:
            raise ImportError from exc

    def add_module(self, fullname: str, is_package: bool, filename: str, source_text: str, source_hash: str) -> None:
        # pylint: disable=too-many-arguments
        self._registered_modules[fullname] = self.RegisteredModule(fullname, is_package, filename, source_text, source_hash)

    def enable_importer(self) -> None:
        insert_loc = 0
        sys_meta_path = self.sys.meta_path
        for idx, obj in enumerate(sys_meta_path):
            name = getattr(obj, "__name__", obj.__class__.__name__)
            if name in ["BuiltinImporter", "FrozenImporter"]:
                insert_loc = idx + 1
        sys_meta_path.insert(insert_loc, self)

    def cleanup_importer(self):
        # sys.meta_path and sys.modules don't need to be restored except where the script may be run multiple times via eval/exec.
        # For that case, _EmbeddedImporterRestoreSys, or an importer that subclasses it, should be used.
        pass
