# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

"""
Custom types for embedding code
"""

# pylint: disable=missing-class-docstring

from dataclasses import dataclass
from pathlib import Path
from typing import Dict, List, Optional
from collections.abc import Callable

SourceText = List[str]
ModuleSourceSearchFunc = Callable[[str], Optional[Path]]


@dataclass
class ModuleSource:
    text: SourceText
    path: Optional[Path] = None
    module_fullname: Optional[str] = None
    text_hash: Optional[str] = None


@dataclass
class ImportedModule:
    fullname: str
    import_source: ModuleSource
    import_line: int
    path: Optional[Path] = None  # TODO: expand to a list of Paths so that compiled and extension modules can also be included
    source: Optional[ModuleSource] = None


ImportedModules = Dict[str, ImportedModule]
