# Copyright (C) 2024, UChicago Argonne, LLC
# Licensed under the 3-clause BSD license.  See accompanying LICENSE.txt file
# in the top-level directory.

import os
import lockfile
import daemon
import signal
from Octeres.util import time_pid_print


def get_pidfile(output_name, output_path):
    pidfile_path = os.path.join(output_path, f"{output_name}.pid")
    pidfile = lockfile.FileLock(pidfile_path)
    return pidfile


def run_daemon(daemon_main, output_name, output_path, pidfile, *args, **kwargs):
    logfile_path = os.path.join(output_path, f"{output_name}.log")
    time_pid_print(f"running {output_name} as a daemon.  Outputting to {output_path}")
    time_pid_print(f"outputting log to {logfile_path}")
    try:
        # FIXME: use a logger and get the date to roll.
        with open(logfile_path, "a") as fd:
            context = daemon.DaemonContext(working_directory=output_path, pidfile=pidfile, stdout=fd, stderr=fd)
            context.signal_map = {
                signal.SIGTERM: "terminate",
                signal.SIGHUP: "terminate",
                signal.SIGUSR1: "terminate",
            }
            time_pid_print(context.pidfile)
            with context:
                daemon_main(*args, **kwargs)
    except lockfile.LockTimeout:
        time_pid_print("file lock already exists.")
